window.dundasVennLib = {
	
	
	// call this during the ready event to create a new Venn diagram
	// html label: 			the html label containing the div where the chart will be shown
	// div: 				the div where the diagram will be shown
	
	// metricSetId: 		the MetricSet that the Venn Diagram will take it's data from
	// categoryHierarchy: 	the hierarchy in the MetricSet that will identify the different categories
	// memberMeasure: 		the measure that will determine the size of the different sections
	
	// viewParameterMappings (Optional): an array of mappings from view parameters to elements in the metricSet
		// Mapping Format: {targetElementName: "NameOfElement", parameter: aViewParameter}
	
	generateVenn: function(htmlLabel, div,
							metricSetId, categoryHierarchy, memberMeasure,
							viewParameterMappings)
	{
		

		viewParameterMappings = viewParameterMappings || [];
	
		
	
	
		// use the htmlLabel's width and height
		var width = htmlLabel.width;
		var height = htmlLabel.height;
		
		// initialize the chart
		var chart = venn.VennDiagram()
		
		
		
		// create a function on the div to refresh the diagram
		htmlLabel.refreshVenn = function()
		{
		
			window.dundasVennLib.getVennData(metricSetId, categoryHierarchy, memberMeasure, div, viewParameterMappings)
			.done(function()
			{
				chart.width(htmlLabel.width);
				chart.height(htmlLabel.height);
				
				var sets = div.sets;
				
				div.datum(div.sets).call(chart);

				var tooltip = d3.select("body").append("div")
					.attr("class", "venntooltip");

				div.selectAll("path")
					.style("stroke-opacity", 0)
					.style("stroke", "#fff")
					.style("stroke-width", 3)

				div.selectAll("g")
					.on("mouseover", function(d, i) {
						// sort all the areas relative to the current item
						venn.sortAreas(div, d);

						// Display a tooltip with the current size
						tooltip.transition().duration(400).style("opacity", .9);
						tooltip.text(memberMeasure + ": " + d.size);

						// highlight the current path
						var selection = d3.select(this).transition("tooltip").duration(400);
						selection.select("path")
							.style("fill-opacity", d.sets.length == 1 ? .4 : .1)
							.style("stroke-opacity", 1);
					})

					.on("mousemove", function() {
						tooltip.style("left", (d3.event.pageX) + "px")
							   .style("top", (d3.event.pageY - 28) + "px");
					})

					.on("mouseout", function(d, i) {
						tooltip.transition().duration(400).style("opacity", 0);
						var selection = d3.select(this).transition("tooltip").duration(400);
						selection.select("path")
							.style("fill-opacity", d.sets.length == 1 ? .25 : .0)
							.style("stroke-opacity", 0);
					});
				
				
			});
				
			
			
		};	
			
		
		
		htmlLabel.refreshVenn();
			
		
	},
	
	
	
	
	// gets data for a Venn diagram
	
	// metricSetId: id of the MetricSet to use
	// categoryHierarchy: name of the hierarchy that contains the category names
	// memberMeasure: name of the measure that determines the size of the diagram sections
	
	// div: the div where the venn diagram is drawn
	// viewParameterMappings: the list of mappings for parameter values
	
	getVennData: function(metricSetId, categoryHierarchy, memberMeasure, div, viewParameterMappings)
	{
		// create a new deferred to signal when done
		var deferred = new $.Deferred();
		
		// get the metric set to use
		var metricSetService = dundas.context.getService("MetricSetService");
		metricSetService.getMetricSetById(metricSetId).done(
			function(metricSet)
			{
				
				
		
				
				// Request data from a metric set by its ID:
				var request = new dundas.data.Request({ objectId: metricSetId });
				// Do not limit the number of rows by paging:
				request.pagingOptions.pagingKind = dundas.data.PagingKind.NONE;
				
				
				// for each view parameter mapping, add a new parameter value to the request
				viewParameterMappings.forEach(function(mapping)
				{
					var targetElement = metricSet.elements.filter(function(element){
						return element.uniqueName === mapping.targetElementName; 
					})[0];
					
					
					var newVal = $.extend({}, mapping.parameter.parameterValue);
					
					newVal.parameterId = targetElement.parameter.id;
					
					request.parameterValues.push(newVal);
					
				});
				
				
				// Get the data retrieval service
				var dataRetrievalService = dundas.context.getService("DataRetrievalService");
				// Make this aysnc
				var dataPromise = dataRetrievalService.getData(request);
				// Used to show loading animation over the bing map during the data request
				var viewService = dundas.context.getService("ViewService");
				viewService.showLoadingRestCall(dataPromise, { "element": div});
				// On data retrieval successful

				

				// once the data arrives, convert it into the set structure used by Venn.js
				dataPromise.done(function (results) 
				{

					div[0][0].removeChild(div[0][0].childNodes[0]);
				
					var myResult = results.filter(function(result){return result.clientData === request.clientData;})[0];
				
					div.sets = []
					
					var cellset = myResult.cellset;
					
			
					// find the column ordinal for the member measure
					var columnOrdinal = 0;
					
					cellset.columns.forEach(function(column)
					{
						if(column.members != null)
						{
							var filteredMembers = column.members.filter(function(member)
							{
								return member.uniqueName === memberMeasure;
							});
								
							if(filteredMembers.length > 0)
							{
								columnOrdinal = column.ordinal;
							}
						}
					});
					
					
					// read the categoryHierarchy value and memberMeasure value from each row, and add a new set entry to the sets
					cellset.rows.forEach(function(row)
					{
						
						
						var setEntry = {"sets":[], "size": 0};
						
						row.members.forEach(function(member)
						{
							
							if(member.hierarchyUniqueName === categoryHierarchy)
							{
								var rowSets = member.caption.split(" & ");
								setEntry.sets = rowSets;
								if(rowSets.length == 1)
									setEntry.label = rowSets[0];
							}
						});
						
						setEntry.size = cellset.cells[columnOrdinal][row.ordinal].value;
						
						div.sets.push(setEntry);
						
					});
					
					// resolve the deferred
					deferred.resolve();		
				});
				
			});
			
		

		// return a promise
		return deferred.promise();
	}
	
	
	
};